#include <openssl/fips.h>

#include "crypto/fips.h"
#include "jitterentropy.h"

static struct rand_data* ec = NULL;

struct rand_data* FIPS_entropy_init(void)
{
    if (ec != NULL)
        /* Entropy source has been initiated and collector allocated */
        return ec;
    /* If the initialization is successful, the call returns with 0 */
    if (jent_entropy_init_ex(1, JENT_FORCE_FIPS) == 0)
        /* Allocate entropy collector */
        ec = jent_entropy_collector_alloc(1, JENT_FORCE_FIPS);
    return ec;
}

void FIPS_entropy_cleanup(void)
{
    /* Free entropy collector */
    if (ec != NULL) {
        jent_entropy_collector_free(ec);
        ec = NULL;
    }
}

ssize_t FIPS_jitter_entropy(unsigned char *buf, size_t buflen)
{
    ssize_t ent_bytes = -1;

    if (buf != NULL && buflen != 0 && FIPS_entropy_init()) {
        /* Get entropy */
        ent_bytes = jent_read_entropy_safe(&ec, (char *)buf, buflen);
    }
    return ent_bytes;
}
